@@ -1383,7 +1383,7 @@ void partitions(mpz_t npart, UV n)
 }
 
 
-void _GMP_lcm_of_consecutive_integers(UV B, mpz_t m)
+void consecutive_integer_lcm(mpz_t m, UV B)
 {
   UV i, p, p_power, pmin;
   mpz_t t[8];
@@ -1444,26 +1444,32 @@ UV is_power(mpz_t n, UV a)
   }
 }
 
-void exp_mangoldt(mpz_t res, mpz_t n)
+UV prime_power(mpz_t prime, mpz_t n)
 {
   UV k;
-  mpz_set_ui(res, 1);
-  if (mpz_cmp_ui(n, 1) <= 0)
-    return;
-  k = mpz_scan1(n, 0);
-  if (k > 0) {
-    if (k+1 == mpz_sizeinbase(n, 2))
-      mpz_set_ui(res, 2);
-    return;
+  if (mpz_even_p(n)) {
+    k = mpz_scan1(n, 0);
+    if (k+1 == mpz_sizeinbase(n, 2)) {
+      mpz_set_ui(prime, 2);
+      return k;
+    }
+    return 0;
   }
   if (_GMP_is_prob_prime(n)) {
-    mpz_set(res, n);
-    return;
+    mpz_set(prime, n);
+    return 1;
   }
-  k = power_factor(n, res);
-  if (k > 1 && _GMP_is_prob_prime(res))
-    return;
-  mpz_set_ui(res, 1);
+  k = power_factor(n, prime);
+  if (k && !_GMP_is_prob_prime(prime))
+    k = 0;
+  return k;
+}
+
+
+void exp_mangoldt(mpz_t res, mpz_t n)
+{
+  if (prime_power(res, n) < 1)
+    mpz_set_ui(res, 1);
 }
 
 
@@ -1735,6 +1741,75 @@ char* pidigits(UV n) {
   return out;
 }
 
+/*****************************************************************************/
+
+void count_primes(mpz_t count, mpz_t lo, mpz_t hi)
+{
+  uint32_t* comp;
+  UV i, cnt, hbits, depth, width;
+  mpz_t shi, t;
+
+  mpz_set_ui(count, 0);
+  if (mpz_cmp_ui(lo,2) <= 0) {
+    if (mpz_cmp_ui(hi,2) >= 0) mpz_add_ui(count,count,1);
+    mpz_set_ui(lo,3);
+  }
+  if (mpz_cmp(lo,hi) > 0) return;
+
+  mpz_init(t);
+  if (mpz_add_ui(t, lo, 100000), mpz_cmp(t,hi) > 0) {
+    mpz_sub_ui(lo,lo,1);
+    for (cnt = 0; mpz_cmp(lo, hi) <= 0; _GMP_next_prime(lo))
+      cnt++;
+    mpz_add_ui(count,count,cnt-1);
+    mpz_clear(t);
+    return;
+  }
+
+  /* Determine a reasonable depth for sieve. */
+  hbits = mpz_sizeinbase(hi,2);
+  depth = (hbits < 100) ? 50000000UL : ((UV)hbits)*UVCONST(500000);
+  if (hbits < 64) {
+    mpz_sqrt(t, hi);
+    if (mpz_cmp_ui(t,depth) < 0)
+      depth = mpz_get_ui(t);
+  }
+
+  /* Count small inputs directly.  We should do this faster. */
+  if (mpz_cmp_ui(lo, depth) <= 0) {
+    mpz_sub_ui(lo,lo,1);
+    for (cnt = 0; mpz_cmp_ui(lo, depth) <= 0; _GMP_next_prime(lo))
+      cnt++;
+    mpz_add_ui(count,count,cnt-1);
+  }
+
+  /* Ensure endpoints are odd */
+  if (mpz_even_p(lo)) mpz_add_ui(lo,lo,1);
+  if (mpz_even_p(hi)) mpz_sub_ui(hi,hi,1);
+
+  mpz_init(shi);
+  while (mpz_cmp(lo,hi) <= 0) {
+    mpz_add_ui(shi, lo, 1000000000UL);
+    if (mpz_cmp(shi, hi) > 0)
+      mpz_set(shi, hi);
+    mpz_sub(t, shi, lo);
+    width = mpz_get_ui(t) + 1;
+
+    comp = partial_sieve(lo, width, depth);
+    for (i = 1, cnt = 0; i <= width; i += 2) {
+      if (!TSTAVAL(comp, i) && (mpz_add_ui(t, lo, i), _GMP_BPSW(t)))
+        cnt++;
+    }
+    Safefree(comp);
+
+    mpz_add_ui(lo, shi, 2);
+    mpz_add_ui(count, count, cnt);
+  }
+  mpz_clear(shi);
+  mpz_clear(t);
+}
+
+
 typedef struct {
   uint32_t nmax;
   uint32_t nsize;
@@ -1941,7 +2016,7 @@ UV* sieve_cluster(mpz_t low, mpz_t high, uint32_t* cl, UV nc, UV *rn) {
   uint32_t pi, startpi = 1;
   uint32_t lastspr = sprimes[maxpi-1];
   uint32_t c, smallnc;
-  UV ibase = 0, nprps = 0;
+  UV ibase = 0, num_mr = 0, num_lucas = 0;
   char crem_0[53*59], crem_1[61*67], crem_2[71*73], *VPrem;
   int run_pretests = 0;
   int _verbose = get_verbose_level();
@@ -2037,7 +2112,7 @@ UV* sieve_cluster(mpz_t low, mpz_t high, uint32_t* cl, UV nc, UV *rn) {
   }
 
   mpz_init_set(savelow, low);
-  if (mpz_sizeinbase(low, 2) > 260) run_pretests = 1;
+  if (mpz_sizeinbase(low, 2) > 310) run_pretests = 1;
   if (run_pretests && mpz_sgn(_bgcd2) == 0) {
     _GMP_pn_primorial(_bgcd2, BGCD2_PRIMES);
     mpz_divexact(_bgcd2, _bgcd2, _bgcd);
@@ -2162,9 +2237,12 @@ UV* sieve_cluster(mpz_t low, mpz_t high, uint32_t* cl, UV nc, UV *rn) {
           if (mpz_add_ui(t, low, i+cl[c]), mpz_gcd(t,t,_bgcd2), mpz_cmp_ui(t,1)) break;
         if (c != nc) continue;
       }
-      /* PRP test */
+      /* PRP test.  Split BPSW in two for faster rejection. */
       for (c = 0; c < nc; c++)
-        if (! (mpz_add_ui(t, low, i+cl[c]), nprps++, _GMP_BPSW(t)) ) break;
+        if (! (mpz_add_ui(t, low, i+cl[c]), num_mr++, miller_rabin_ui(t,2)) ) break;
+      if (c != nc) continue;
+      for (c = 0; c < nc; c++)
+        if (! (mpz_add_ui(t, low, i+cl[c]), num_lucas++, _GMP_is_lucas_pseudoprime(t,2)) ) break;
       if (c != nc) continue;
       PUSH_VLIST(retlist, ibase + i);
     }
@@ -2172,7 +2250,7 @@ UV* sieve_cluster(mpz_t low, mpz_t high, uint32_t* cl, UV nc, UV *rn) {
     mpz_add_ui(low, low, ppr);
   }
 
-  if (_verbose) printf("cluster sieve ran %"UVuf" BPSW tests (pretests %s)\n", nprps, run_pretests ? "on" : "off");
+  if (_verbose) printf("cluster sieve ran %"UVuf" MR and %"UVuf" Lucas tests (pretests %s)\n", num_mr, num_lucas, run_pretests ? "on" : "off");
   mpz_set(low, savelow);
   Safefree(cres);
   Safefree(VPrem);
