@@ -15,7 +15,7 @@ extern void surround_primes(mpz_t n, UV* prev, UV* next, UV skip_width);
 
 extern void _GMP_pn_primorial(mpz_t prim, UV n);
 extern void _GMP_primorial(mpz_t prim, UV n);
-extern void _GMP_lcm_of_consecutive_integers(UV B, mpz_t m);
+extern void consecutive_integer_lcm(mpz_t m, UV B);
 extern void bernfrac(mpz_t num, mpz_t den, mpz_t n);
 extern void harmfrac(mpz_t num, mpz_t den, mpz_t n);
 extern void stirling(mpz_t r, unsigned long n, unsigned long m, UV type);
@@ -24,6 +24,7 @@ extern void partitions(mpz_t npart, UV n);
 
 extern UV   is_power(mpz_t n, UV a);
 
+extern UV   prime_power(mpz_t prime, mpz_t n);
 extern void exp_mangoldt(mpz_t res, mpz_t n);
 
 extern uint32_t* partial_sieve(mpz_t start, UV length, UV maxprime);
@@ -34,6 +35,7 @@ extern char* zetareal(mpf_t r, unsigned long prec);
 extern char* riemannrreal(mpf_t r, unsigned long prec);
 extern char* lambertwreal(mpf_t r, unsigned long prec);
 
+extern void count_primes(mpz_t count, mpz_t lo, mpz_t hi);
 extern UV* sieve_primes(mpz_t low, mpz_t high, UV k, UV *rn);
 extern UV* sieve_twin_primes(mpz_t low, mpz_t high, UV twin, UV *rn);
 extern UV* sieve_cluster(mpz_t low, mpz_t high, uint32_t* cl, UV nc, UV *rn);
