@@ -1,8 +1,7 @@
 #ifndef MPU_PITERATOR_H
 #define MPU_PITERATOR_H
 
-#include "EXTERN.h"
-#include "perl.h"
+#include "ptypes.h"
 
 typedef struct {
   UV p;
@@ -21,4 +20,6 @@ extern UV prime_iterator_next(prime_iterator *iter);
 extern void prime_iterator_setprime(prime_iterator *iter, UV n);
 extern int prime_iterator_isprime(prime_iterator *iter, UV n);
 
+extern UV* sieve_to_n(UV n, UV* count);
+
 #endif
